@@ -10,28 +10,45 @@ Link: https://github.com/blei-lab/onlineldavb
 
 # Author: Chyi-Kwei Yau
 # Author: Matthew D. Hoffman (original onlineldavb implementation)
+from numbers import Integral, Real
 
 import numpy as np
 import scipy.sparse as sp
+from joblib import effective_n_jobs
 from scipy.special import gammaln, logsumexp
-from joblib import Parallel, effective_n_jobs
 
-from ..base import BaseEstimator, TransformerMixin
+from ..base import (
+    BaseEstimator,
+    ClassNamePrefixFeaturesOutMixin,
+    TransformerMixin,
+    _fit_context,
+)
 from ..utils import check_random_state, gen_batches, gen_even_slices
-from ..utils.validation import check_non_negative
-from ..utils.validation import check_is_fitted
-from ..utils.validation import _deprecate_positional_args
-from ..utils.fixes import delayed
-
-from ._online_lda_fast import (mean_change, _dirichlet_expectation_1d,
-                               _dirichlet_expectation_2d)
+from ..utils._param_validation import Interval, StrOptions
+from ..utils.parallel import Parallel, delayed
+from ..utils.validation import check_is_fitted, check_non_negative
+from ._online_lda_fast import (
+    _dirichlet_expectation_1d as cy_dirichlet_expectation_1d,
+)
+from ._online_lda_fast import (
+    _dirichlet_expectation_2d,
+)
+from ._online_lda_fast import (
+    mean_change as cy_mean_change,
+)
 
 EPS = np.finfo(float).eps
 
 
-def _update_doc_distribution(X, exp_topic_word_distr, doc_topic_prior,
-                             max_iters,
-                             mean_change_tol, cal_sstats, random_state):
+def _update_doc_distribution(
+    X,
+    exp_topic_word_distr,
+    doc_topic_prior,
+    max_doc_update_iter,
+    mean_change_tol,
+    cal_sstats,
+    random_state,
+):
     """E-step: update document-topic distribution.
 
     Parameters
@@ -46,12 +63,12 @@ def _update_doc_distribution(X, exp_topic_word_distr, doc_topic_prior,
     doc_topic_prior : float
         Prior of document topic distribution `theta`.
 
-    max_iters : int
+    max_doc_update_iter : int
         Max number of iterations for updating document topic distribution in
         the E-step.
 
     mean_change_tol : float
-        Stopping tolerance for updating document topic distribution in E-setp.
+        Stopping tolerance for updating document topic distribution in E-step.
 
     cal_sstats : bool
         Parameter that indicate to calculate sufficient statistics or not.
@@ -77,25 +94,38 @@ def _update_doc_distribution(X, exp_topic_word_distr, doc_topic_prior,
     n_topics = exp_topic_word_distr.shape[0]
 
     if random_state:
-        doc_topic_distr = random_state.gamma(100., 0.01, (n_samples, n_topics))
+        doc_topic_distr = random_state.gamma(100.0, 0.01, (n_samples, n_topics)).astype(
+            X.dtype, copy=False
+        )
     else:
-        doc_topic_distr = np.ones((n_samples, n_topics))
+        doc_topic_distr = np.ones((n_samples, n_topics), dtype=X.dtype)
 
     # In the literature, this is `exp(E[log(theta)])`
     exp_doc_topic = np.exp(_dirichlet_expectation_2d(doc_topic_distr))
 
     # diff on `component_` (only calculate it when `cal_diff` is True)
-    suff_stats = np.zeros(exp_topic_word_distr.shape) if cal_sstats else None
+    suff_stats = (
+        np.zeros(exp_topic_word_distr.shape, dtype=X.dtype) if cal_sstats else None
+    )
 
     if is_sparse_x:
         X_data = X.data
         X_indices = X.indices
         X_indptr = X.indptr
 
+    # These cython functions are called in a nested loop on usually very small arrays
+    # (length=n_topics). In that case, finding the appropriate signature of the
+    # fused-typed function can be more costly than its execution, hence the dispatch
+    # is done outside of the loop.
+    ctype = "float" if X.dtype == np.float32 else "double"
+    mean_change = cy_mean_change[ctype]
+    dirichlet_expectation_1d = cy_dirichlet_expectation_1d[ctype]
+    eps = np.finfo(X.dtype).eps
+
     for idx_d in range(n_samples):
         if is_sparse_x:
-            ids = X_indices[X_indptr[idx_d]:X_indptr[idx_d + 1]]
-            cnts = X_data[X_indptr[idx_d]:X_indptr[idx_d + 1]]
+            ids = X_indices[X_indptr[idx_d] : X_indptr[idx_d + 1]]
+            cnts = X_data[X_indptr[idx_d] : X_indptr[idx_d + 1]]
         else:
             ids = np.nonzero(X[idx_d, :])[0]
             cnts = X[idx_d, ids]
@@ -106,18 +136,16 @@ def _update_doc_distribution(X, exp_topic_word_distr, doc_topic_prior,
         exp_topic_word_d = exp_topic_word_distr[:, ids]
 
         # Iterate between `doc_topic_d` and `norm_phi` until convergence
-        for _ in range(0, max_iters):
+        for _ in range(0, max_doc_update_iter):
             last_d = doc_topic_d
 
             # The optimal phi_{dwk} is proportional to
             # exp(E[log(theta_{dk})]) * exp(E[log(beta_{dw})]).
-            norm_phi = np.dot(exp_doc_topic_d, exp_topic_word_d) + EPS
+            norm_phi = np.dot(exp_doc_topic_d, exp_topic_word_d) + eps
 
-            doc_topic_d = (exp_doc_topic_d *
-                           np.dot(cnts / norm_phi, exp_topic_word_d.T))
+            doc_topic_d = exp_doc_topic_d * np.dot(cnts / norm_phi, exp_topic_word_d.T)
             # Note: adds doc_topic_prior to doc_topic_d, in-place.
-            _dirichlet_expectation_1d(doc_topic_d, doc_topic_prior,
-                                      exp_doc_topic_d)
+            dirichlet_expectation_1d(doc_topic_d, doc_topic_prior, exp_doc_topic_d)
 
             if mean_change(last_d, doc_topic_d) < mean_change_tol:
                 break
@@ -126,14 +154,18 @@ def _update_doc_distribution(X, exp_topic_word_distr, doc_topic_prior,
         # Contribution of document d to the expected sufficient
         # statistics for the M step.
         if cal_sstats:
-            norm_phi = np.dot(exp_doc_topic_d, exp_topic_word_d) + EPS
+            norm_phi = np.dot(exp_doc_topic_d, exp_topic_word_d) + eps
             suff_stats[:, ids] += np.outer(exp_doc_topic_d, cnts / norm_phi)
 
     return (doc_topic_distr, suff_stats)
 
 
-class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
-    """Latent Dirichlet Allocation with online variational Bayes algorithm
+class LatentDirichletAllocation(
+    ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator
+):
+    """Latent Dirichlet Allocation with online variational Bayes algorithm.
+
+    The implementation is based on [1]_ and [2]_.
 
     .. versionadded:: 0.17
 
@@ -182,19 +214,21 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         ``n_samples``, the update method is same as batch learning. In the
         literature, this is called kappa.
 
-    learning_offset : float, default=10.
+    learning_offset : float, default=10.0
         A (positive) parameter that downweights early iterations in online
         learning.  It should be greater than 1.0. In the literature, this is
         called tau_0.
 
     max_iter : int, default=10
-        The maximum number of iterations.
+        The maximum number of passes over the training data (aka epochs).
+        It only impacts the behavior in the :meth:`fit` method, and not the
+        :meth:`partial_fit` method.
 
     batch_size : int, default=128
         Number of documents to use in each EM iteration. Only used in online
         learning.
 
-    evaluate_every : int, default=0
+    evaluate_every : int, default=-1
         How often to evaluate perplexity. Only used in `fit` method.
         set it to 0 or negative number to not evaluate perplexity in
         training at all. Evaluating perplexity can help you check convergence
@@ -247,6 +281,17 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
     n_batch_iter_ : int
         Number of iterations of the EM step.
 
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
     n_iter_ : int
         Number of passes over the dataset.
 
@@ -265,6 +310,21 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         Prior of topic word distribution `beta`. If the value is None, it is
         `1 / n_components`.
 
+    See Also
+    --------
+    sklearn.discriminant_analysis.LinearDiscriminantAnalysis:
+        A classifier with a linear decision boundary, generated by fitting
+        class conditional densities to the data and using Bayes' rule.
+
+    References
+    ----------
+    .. [1] "Online Learning for Latent Dirichlet Allocation", Matthew D.
+           Hoffman, David M. Blei, Francis Bach, 2010
+           https://github.com/blei-lab/onlineldavb
+
+    .. [2] "Stochastic Variational Inference", Matthew D. Hoffman,
+           David M. Blei, Chong Wang, John Paisley, 2013
+
     Examples
     --------
     >>> from sklearn.decomposition import LatentDirichletAllocation
@@ -280,26 +340,47 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
     >>> lda.transform(X[-2:])
     array([[0.00360392, 0.25499205, 0.0036211 , 0.64236448, 0.09541846],
            [0.15297572, 0.00362644, 0.44412786, 0.39568399, 0.003586  ]])
-
-    References
-    ----------
-    .. [1] "Online Learning for Latent Dirichlet Allocation", Matthew D.
-        Hoffman, David M. Blei, Francis Bach, 2010
-
-    [2] "Stochastic Variational Inference", Matthew D. Hoffman, David M. Blei,
-        Chong Wang, John Paisley, 2013
-
-    [3] Matthew D. Hoffman's onlineldavb code. Link:
-        https://github.com/blei-lab/onlineldavb
-
     """
-    @_deprecate_positional_args
-    def __init__(self, n_components=10, *, doc_topic_prior=None,
-                 topic_word_prior=None, learning_method='batch',
-                 learning_decay=.7, learning_offset=10., max_iter=10,
-                 batch_size=128, evaluate_every=-1, total_samples=1e6,
-                 perp_tol=1e-1, mean_change_tol=1e-3, max_doc_update_iter=100,
-                 n_jobs=None, verbose=0, random_state=None):
+
+    _parameter_constraints: dict = {
+        "n_components": [Interval(Integral, 0, None, closed="neither")],
+        "doc_topic_prior": [None, Interval(Real, 0, 1, closed="both")],
+        "topic_word_prior": [None, Interval(Real, 0, 1, closed="both")],
+        "learning_method": [StrOptions({"batch", "online"})],
+        "learning_decay": [Interval(Real, 0, 1, closed="both")],
+        "learning_offset": [Interval(Real, 1.0, None, closed="left")],
+        "max_iter": [Interval(Integral, 0, None, closed="left")],
+        "batch_size": [Interval(Integral, 0, None, closed="neither")],
+        "evaluate_every": [Interval(Integral, None, None, closed="neither")],
+        "total_samples": [Interval(Real, 0, None, closed="neither")],
+        "perp_tol": [Interval(Real, 0, None, closed="left")],
+        "mean_change_tol": [Interval(Real, 0, None, closed="left")],
+        "max_doc_update_iter": [Interval(Integral, 0, None, closed="left")],
+        "n_jobs": [None, Integral],
+        "verbose": ["verbose"],
+        "random_state": ["random_state"],
+    }
+
+    def __init__(
+        self,
+        n_components=10,
+        *,
+        doc_topic_prior=None,
+        topic_word_prior=None,
+        learning_method="batch",
+        learning_decay=0.7,
+        learning_offset=10.0,
+        max_iter=10,
+        batch_size=128,
+        evaluate_every=-1,
+        total_samples=1e6,
+        perp_tol=1e-1,
+        mean_change_tol=1e-3,
+        max_doc_update_iter=100,
+        n_jobs=None,
+        verbose=0,
+        random_state=None,
+    ):
         self.n_components = n_components
         self.doc_topic_prior = doc_topic_prior
         self.topic_word_prior = topic_word_prior
@@ -317,25 +398,7 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         self.verbose = verbose
         self.random_state = random_state
 
-    def _check_params(self):
-        """Check model parameters."""
-        if self.n_components <= 0:
-            raise ValueError("Invalid 'n_components' parameter: %r"
-                             % self.n_components)
-
-        if self.total_samples <= 0:
-            raise ValueError("Invalid 'total_samples' parameter: %r"
-                             % self.total_samples)
-
-        if self.learning_offset < 0:
-            raise ValueError("Invalid 'learning_offset' parameter: %r"
-                             % self.learning_offset)
-
-        if self.learning_method not in ("batch", "online"):
-            raise ValueError("Invalid 'learning_method' parameter: %r"
-                             % self.learning_method)
-
-    def _init_latent_vars(self, n_features):
+    def _init_latent_vars(self, n_features, dtype=np.float64):
         """Initialize latent variables."""
 
         self.random_state_ = check_random_state(self.random_state)
@@ -343,24 +406,26 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         self.n_iter_ = 0
 
         if self.doc_topic_prior is None:
-            self.doc_topic_prior_ = 1. / self.n_components
+            self.doc_topic_prior_ = 1.0 / self.n_components
         else:
             self.doc_topic_prior_ = self.doc_topic_prior
 
         if self.topic_word_prior is None:
-            self.topic_word_prior_ = 1. / self.n_components
+            self.topic_word_prior_ = 1.0 / self.n_components
         else:
             self.topic_word_prior_ = self.topic_word_prior
 
-        init_gamma = 100.
-        init_var = 1. / init_gamma
+        init_gamma = 100.0
+        init_var = 1.0 / init_gamma
         # In the literature, this is called `lambda`
         self.components_ = self.random_state_.gamma(
-            init_gamma, init_var, (self.n_components, n_features))
+            init_gamma, init_var, (self.n_components, n_features)
+        ).astype(dtype, copy=False)
 
         # In the literature, this is `exp(E[log(beta)])`
         self.exp_dirichlet_component_ = np.exp(
-            _dirichlet_expectation_2d(self.components_))
+            _dirichlet_expectation_2d(self.components_)
+        )
 
     def _e_step(self, X, cal_sstats, random_init, parallel=None):
         """E-step in EM update.
@@ -398,16 +463,19 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         # TODO: make Parallel._effective_n_jobs public instead?
         n_jobs = effective_n_jobs(self.n_jobs)
         if parallel is None:
-            parallel = Parallel(n_jobs=n_jobs, verbose=max(0,
-                                                           self.verbose - 1))
+            parallel = Parallel(n_jobs=n_jobs, verbose=max(0, self.verbose - 1))
         results = parallel(
-            delayed(_update_doc_distribution)(X[idx_slice, :],
-                                              self.exp_dirichlet_component_,
-                                              self.doc_topic_prior_,
-                                              self.max_doc_update_iter,
-                                              self.mean_change_tol, cal_sstats,
-                                              random_state)
-            for idx_slice in gen_even_slices(X.shape[0], n_jobs))
+            delayed(_update_doc_distribution)(
+                X[idx_slice, :],
+                self.exp_dirichlet_component_,
+                self.doc_topic_prior_,
+                self.max_doc_update_iter,
+                self.mean_change_tol,
+                cal_sstats,
+                random_state,
+            )
+            for idx_slice in gen_even_slices(X.shape[0], n_jobs)
+        )
 
         # merge result
         doc_topics, sstats_list = zip(*results)
@@ -416,7 +484,7 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         if cal_sstats:
             # This step finishes computing the sufficient statistics for the
             # M-step.
-            suff_stats = np.zeros(self.components_.shape)
+            suff_stats = np.zeros(self.components_.shape, dtype=self.components_.dtype)
             for sstats in sstats_list:
                 suff_stats += sstats
             suff_stats *= self.exp_dirichlet_component_
@@ -453,8 +521,9 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         """
 
         # E-step
-        _, suff_stats = self._e_step(X, cal_sstats=True, random_init=True,
-                                     parallel=parallel)
+        _, suff_stats = self._e_step(
+            X, cal_sstats=True, random_init=True, parallel=parallel
+        )
 
         # M-step
         if batch_update:
@@ -462,21 +531,27 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         else:
             # online update
             # In the literature, the weight is `rho`
-            weight = np.power(self.learning_offset + self.n_batch_iter_,
-                              -self.learning_decay)
+            weight = np.power(
+                self.learning_offset + self.n_batch_iter_, -self.learning_decay
+            )
             doc_ratio = float(total_samples) / X.shape[0]
-            self.components_ *= (1 - weight)
-            self.components_ += (weight * (self.topic_word_prior_
-                                           + doc_ratio * suff_stats))
+            self.components_ *= 1 - weight
+            self.components_ += weight * (
+                self.topic_word_prior_ + doc_ratio * suff_stats
+            )
 
         # update `component_` related variables
         self.exp_dirichlet_component_ = np.exp(
-            _dirichlet_expectation_2d(self.components_))
+            _dirichlet_expectation_2d(self.components_)
+        )
         self.n_batch_iter_ += 1
         return
 
     def _more_tags(self):
-        return {'requires_positive_X': True}
+        return {
+            "preserves_dtype": [np.float64, np.float32],
+            "requires_positive_X": True,
+        }
 
     def _check_non_neg_array(self, X, reset_n_features, whom):
         """check X format
@@ -488,11 +563,19 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         X :  array-like or sparse matrix
 
         """
-        X = self._validate_data(X, reset=reset_n_features,
-                                accept_sparse='csr')
+        dtype = [np.float64, np.float32] if reset_n_features else self.components_.dtype
+
+        X = self._validate_data(
+            X,
+            reset=reset_n_features,
+            accept_sparse="csr",
+            dtype=dtype,
+        )
         check_non_negative(X, whom)
+
         return X
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def partial_fit(self, X, y=None):
         """Online VB with Mini-Batch update.
 
@@ -502,40 +585,45 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
             Document word matrix.
 
         y : Ignored
+            Not used, present here for API consistency by convention.
 
         Returns
         -------
         self
+            Partially fitted estimator.
         """
-        self._check_params()
-        first_time = not hasattr(self, 'components_')
+        first_time = not hasattr(self, "components_")
+
         X = self._check_non_neg_array(
-            X, reset_n_features=first_time,
-            whom="LatentDirichletAllocation.partial_fit")
+            X, reset_n_features=first_time, whom="LatentDirichletAllocation.partial_fit"
+        )
         n_samples, n_features = X.shape
         batch_size = self.batch_size
 
         # initialize parameters or check
         if first_time:
-            self._init_latent_vars(n_features)
+            self._init_latent_vars(n_features, dtype=X.dtype)
 
         if n_features != self.components_.shape[1]:
             raise ValueError(
                 "The provided data has %d dimensions while "
-                "the model was trained with feature size %d." %
-                (n_features, self.components_.shape[1]))
+                "the model was trained with feature size %d."
+                % (n_features, self.components_.shape[1])
+            )
 
         n_jobs = effective_n_jobs(self.n_jobs)
-        with Parallel(n_jobs=n_jobs,
-                      verbose=max(0, self.verbose - 1)) as parallel:
+        with Parallel(n_jobs=n_jobs, verbose=max(0, self.verbose - 1)) as parallel:
             for idx_slice in gen_batches(n_samples, batch_size):
-                self._em_step(X[idx_slice, :],
-                              total_samples=self.total_samples,
-                              batch_update=False,
-                              parallel=parallel)
+                self._em_step(
+                    X[idx_slice, :],
+                    total_samples=self.total_samples,
+                    batch_update=False,
+                    parallel=parallel,
+                )
 
         return self
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y=None):
         """Learn model for the data X with variational Bayes method.
 
@@ -548,14 +636,16 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
             Document word matrix.
 
         y : Ignored
+            Not used, present here for API consistency by convention.
 
         Returns
         -------
         self
+            Fitted estimator.
         """
-        self._check_params()
-        X = self._check_non_neg_array(X, reset_n_features=True,
-                                      whom="LatentDirichletAllocation.fit")
+        X = self._check_non_neg_array(
+            X, reset_n_features=True, whom="LatentDirichletAllocation.fit"
+        )
         n_samples, n_features = X.shape
         max_iter = self.max_iter
         evaluate_every = self.evaluate_every
@@ -564,47 +654,55 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         batch_size = self.batch_size
 
         # initialize parameters
-        self._init_latent_vars(n_features)
+        self._init_latent_vars(n_features, dtype=X.dtype)
         # change to perplexity later
         last_bound = None
         n_jobs = effective_n_jobs(self.n_jobs)
-        with Parallel(n_jobs=n_jobs,
-                      verbose=max(0, self.verbose - 1)) as parallel:
+        with Parallel(n_jobs=n_jobs, verbose=max(0, self.verbose - 1)) as parallel:
             for i in range(max_iter):
-                if learning_method == 'online':
+                if learning_method == "online":
                     for idx_slice in gen_batches(n_samples, batch_size):
-                        self._em_step(X[idx_slice, :], total_samples=n_samples,
-                                      batch_update=False, parallel=parallel)
+                        self._em_step(
+                            X[idx_slice, :],
+                            total_samples=n_samples,
+                            batch_update=False,
+                            parallel=parallel,
+                        )
                 else:
                     # batch update
-                    self._em_step(X, total_samples=n_samples,
-                                  batch_update=True, parallel=parallel)
+                    self._em_step(
+                        X, total_samples=n_samples, batch_update=True, parallel=parallel
+                    )
 
                 # check perplexity
                 if evaluate_every > 0 and (i + 1) % evaluate_every == 0:
-                    doc_topics_distr, _ = self._e_step(X, cal_sstats=False,
-                                                       random_init=False,
-                                                       parallel=parallel)
-                    bound = self._perplexity_precomp_distr(X, doc_topics_distr,
-                                                           sub_sampling=False)
+                    doc_topics_distr, _ = self._e_step(
+                        X, cal_sstats=False, random_init=False, parallel=parallel
+                    )
+                    bound = self._perplexity_precomp_distr(
+                        X, doc_topics_distr, sub_sampling=False
+                    )
                     if self.verbose:
-                        print('iteration: %d of max_iter: %d, perplexity: %.4f'
-                              % (i + 1, max_iter, bound))
+                        print(
+                            "iteration: %d of max_iter: %d, perplexity: %.4f"
+                            % (i + 1, max_iter, bound)
+                        )
 
                     if last_bound and abs(last_bound - bound) < self.perp_tol:
                         break
                     last_bound = bound
 
                 elif self.verbose:
-                    print('iteration: %d of max_iter: %d' % (i + 1, max_iter))
+                    print("iteration: %d of max_iter: %d" % (i + 1, max_iter))
                 self.n_iter_ += 1
 
         # calculate final perplexity value on train set
-        doc_topics_distr, _ = self._e_step(X, cal_sstats=False,
-                                           random_init=False,
-                                           parallel=parallel)
-        self.bound_ = self._perplexity_precomp_distr(X, doc_topics_distr,
-                                                     sub_sampling=False)
+        doc_topics_distr, _ = self._e_step(
+            X, cal_sstats=False, random_init=False, parallel=parallel
+        )
+        self.bound_ = self._perplexity_precomp_distr(
+            X, doc_topics_distr, sub_sampling=False
+        )
 
         return self
 
@@ -621,21 +719,7 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         doc_topic_distr : ndarray of shape (n_samples, n_components)
             Document topic distribution for X.
         """
-        check_is_fitted(self)
-
-        # make sure feature size is the same in fitted model and in X
-        X = self._check_non_neg_array(
-            X, reset_n_features=True,
-            whom="LatentDirichletAllocation.transform")
-        n_samples, n_features = X.shape
-        if n_features != self.components_.shape[1]:
-            raise ValueError(
-                "The provided data has %d dimensions while "
-                "the model was trained with feature size %d." %
-                (n_features, self.components_.shape[1]))
-
-        doc_topic_distr, _ = self._e_step(X, cal_sstats=False,
-                                          random_init=False)
+        doc_topic_distr, _ = self._e_step(X, cal_sstats=False, random_init=False)
 
         return doc_topic_distr
 
@@ -657,8 +741,8 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         """
         check_is_fitted(self)
         X = self._check_non_neg_array(
-            X, reset_n_features=False,
-            whom="LatentDirichletAllocation.transform")
+            X, reset_n_features=False, whom="LatentDirichletAllocation.transform"
+        )
         doc_topic_distr = self._unnormalized_transform(X)
         doc_topic_distr /= doc_topic_distr.sum(axis=1)[:, np.newaxis]
         return doc_topic_distr
@@ -714,19 +798,21 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         # E[log p(docs | theta, beta)]
         for idx_d in range(0, n_samples):
             if is_sparse_x:
-                ids = X_indices[X_indptr[idx_d]:X_indptr[idx_d + 1]]
-                cnts = X_data[X_indptr[idx_d]:X_indptr[idx_d + 1]]
+                ids = X_indices[X_indptr[idx_d] : X_indptr[idx_d + 1]]
+                cnts = X_data[X_indptr[idx_d] : X_indptr[idx_d + 1]]
             else:
                 ids = np.nonzero(X[idx_d, :])[0]
                 cnts = X[idx_d, ids]
-            temp = (dirichlet_doc_topic[idx_d, :, np.newaxis]
-                    + dirichlet_component_[:, ids])
+            temp = (
+                dirichlet_doc_topic[idx_d, :, np.newaxis] + dirichlet_component_[:, ids]
+            )
             norm_phi = logsumexp(temp, axis=0)
             score += np.dot(cnts, norm_phi)
 
         # compute E[log p(theta | alpha) - log q(theta | gamma)]
-        score += _loglikelihood(doc_topic_prior, doc_topic_distr,
-                                dirichlet_doc_topic, self.n_components)
+        score += _loglikelihood(
+            doc_topic_prior, doc_topic_distr, dirichlet_doc_topic, self.n_components
+        )
 
         # Compensate for the subsampling of the population of documents
         if sub_sampling:
@@ -734,8 +820,9 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
             score *= doc_ratio
 
         # E[log p(beta | eta) - log q (beta | lambda)]
-        score += _loglikelihood(topic_word_prior, self.components_,
-                                dirichlet_component_, n_features)
+        score += _loglikelihood(
+            topic_word_prior, self.components_, dirichlet_component_, n_features
+        )
 
         return score
 
@@ -748,6 +835,7 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
             Document word matrix.
 
         y : Ignored
+            Not used, present here for API consistency by convention.
 
         Returns
         -------
@@ -755,15 +843,15 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
             Use approximate bound as score.
         """
         check_is_fitted(self)
-        X = self._check_non_neg_array(X, reset_n_features=False,
-                                      whom="LatentDirichletAllocation.score")
+        X = self._check_non_neg_array(
+            X, reset_n_features=False, whom="LatentDirichletAllocation.score"
+        )
 
         doc_topic_distr = self._unnormalized_transform(X)
         score = self._approx_bound(X, doc_topic_distr, sub_sampling=False)
         return score
 
-    def _perplexity_precomp_distr(self, X, doc_topic_distr=None,
-                                  sub_sampling=False):
+    def _perplexity_precomp_distr(self, X, doc_topic_distr=None, sub_sampling=False):
         """Calculate approximate perplexity for data X with ability to accept
         precomputed doc_topic_distr
 
@@ -784,19 +872,14 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         score : float
             Perplexity score.
         """
-        check_is_fitted(self)
-
-        X = self._check_non_neg_array(
-            X, reset_n_features=True,
-            whom="LatentDirichletAllocation.perplexity")
-
         if doc_topic_distr is None:
             doc_topic_distr = self._unnormalized_transform(X)
         else:
             n_samples, n_components = doc_topic_distr.shape
             if n_samples != X.shape[0]:
-                raise ValueError("Number of samples in X and doc_topic_distr"
-                                 " do not match.")
+                raise ValueError(
+                    "Number of samples in X and doc_topic_distr do not match."
+                )
 
             if n_components != self.n_components:
                 raise ValueError("Number of topics does not match.")
@@ -834,4 +917,13 @@ class LatentDirichletAllocation(TransformerMixin, BaseEstimator):
         score : float
             Perplexity score.
         """
+        check_is_fitted(self)
+        X = self._check_non_neg_array(
+            X, reset_n_features=True, whom="LatentDirichletAllocation.perplexity"
+        )
         return self._perplexity_precomp_distr(X, sub_sampling=sub_sampling)
+
+    @property
+    def _n_features_out(self):
+        """Number of transformed output features."""
+        return self.components_.shape[0]
